package net.wanji.web.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import net.wanji.web.common.entity.JsonViewObject;
import net.wanji.web.common.util.StringUtils;
import net.wanji.web.entity.TBaseCrossInfo;
import net.wanji.web.service.SituationDetectionService;
import net.wanji.web.util.HttpRestUtil;
import net.wanji.web.vo.situationDetection.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import javax.ws.rs.core.MediaType;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Objects;

import static net.wanji.web.util.HttpRestUtil.buildHeader;


/**
 * 态势监测
 *
 * @author Kent HAN
 * @date 2022/10/26 13:31
 */
@Api(value = "SituationDetectionController", description = "态势监测")
@RequestMapping("/situationDetection")
@RestController
public class SituationDetectionController extends BaseController {

    @Autowired
    private HttpRestUtil httpRestUtil;

    @Value("${utcServiceUrl}")
    private String utcServiceUrl;
    @Value("${syncDeviceSatusUrn}")
    private String syncDeviceSatusUrl;

    @Autowired
    SituationDetectionService situationDetectionService;

    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    @ApiOperation(value = "优化记录", notes = "优化记录", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/optRecords",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    public JsonViewObject optRecords(@RequestBody OptRecordsInVO optRecordsInVO) throws ParseException {
        String adCode = optRecordsInVO.getAdCode();
        String currentTimeStr = optRecordsInVO.getCurrentTime();
        Date adaptTime = sdf.parse(currentTimeStr);
        OptRecordsOutVO optRecordsOutVO = situationDetectionService.optRecords(adCode, adaptTime);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success(optRecordsOutVO);
    }

    @ApiOperation(value = "路口报警", notes = "路口报警",
            response = JsonViewObject.class, produces = MediaType.APPLICATION_JSON)
    @GetMapping(value = "/crossAlarm", produces = MediaType.APPLICATION_JSON)
    public JsonViewObject crossAlarm() {
        List<CrossAlarmOutVO> crossAlarmOutVOList = situationDetectionService.crossAlarm();
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success(crossAlarmOutVOList);
    }

    @ApiOperation(value = "事件报警", notes = "事件报警", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/eventAlarm",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    public JsonViewObject eventAlarm(@RequestBody EventAlarmInVO eventAlarmInVO) throws ParseException {
        String currentTimeStr = eventAlarmInVO.getCurrentTime();
        Date todayTime = sdf.parse(currentTimeStr);
        List<EventAlarmOutVO> eventAlarmOutVOList = situationDetectionService.eventAlarm(todayTime);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success(eventAlarmOutVOList);
    }

    @ApiOperation(value = "信号控制状态", notes = "信号控制状态", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/controlHist",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    public JsonViewObject controlHist(@RequestBody ControlHistInVO controlHistInVO) throws ParseException {
        String crossId = controlHistInVO.getCrossId();
        String currentTime = controlHistInVO.getCurrentTime();
        List<ControlHistOutVO> controlHistOutVOList = situationDetectionService.controlHist(crossId, currentTime);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success(controlHistOutVOList);
    }

    @ApiOperation(value = "在途车辆", notes = "在途车辆", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/vehicleTraceHist",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    public JsonViewObject vehicleTrace(@RequestBody VehicleTraceInVO vehicleTraceInVO) throws ParseException {
        String adCode = vehicleTraceInVO.getAdCode();
        String currentTime = vehicleTraceInVO.getCurrentTime();
        List<VehicleTraceOutVO> vehicleTraceOutVOList = situationDetectionService.vehicleTrace(adCode, currentTime);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success(vehicleTraceOutVOList);
    }

    @ApiOperation(value = "图层-视频设备信息及状态", notes = "图层-视频设备信息及状态", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/videoDeviceInfo",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    public JsonViewObject videoDeviceInfo(@RequestBody VideoDeviceInfoInVO videoDeviceInfoInVO) {
        String adCode = videoDeviceInfoInVO.getAdCode();
        List<VideoDeviceInfoOutVO> videoDeviceInfoOutVOList = situationDetectionService.videoDeviceInfo(adCode);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success(videoDeviceInfoOutVOList);
    }

    @ApiOperation(value = "绿波监测", notes = "绿波监测", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/greenwaveRealtime",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    public JsonViewObject greenwaveRealtime(@RequestBody GreenwaveRealtimeInVO greenwaveRealtimeInVO) throws ParseException {
        String adCode = greenwaveRealtimeInVO.getAdCode();
        String currentTime = greenwaveRealtimeInVO.getCurrentTime();
        List<GreenwaveRealtimeOutVO> greenwaveRealtimeOutVOList = situationDetectionService.greenwaveRealtime(adCode, currentTime);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success(greenwaveRealtimeOutVOList);
    }

    @ApiOperation(value = "交通设备状态", notes = "交通设备状态", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/allDeviceStatus",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    public JsonViewObject allDeviceStatus(@RequestBody AllDeviceStatusInVO allDeviceStatusInVO) {
        String adCode = allDeviceStatusInVO.getAdCode();
        Map<String, Map<String, Integer>> allDeviceStatusMap = situationDetectionService.allDeviceStatus(adCode);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success(allDeviceStatusMap);
    }

    @ApiOperation(value = "绿波监测-关联路口", notes = "绿波监测-关联路口", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/greenwaveCross",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    public JsonViewObject greenwaveCross(@RequestBody GreenwaveCrossInVO greenwaveCrossInVO) {
        String adCode = greenwaveCrossInVO.getAdCode();
        String currentTime = greenwaveCrossInVO.getCurrentTime();
        Integer id = greenwaveCrossInVO.getId();
        List<GreenwaveCrossOutVO> greenwaveCrossOutVOList = situationDetectionService.greenwaveCross(adCode, currentTime, id);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success(greenwaveCrossOutVOList);
    }

    @ApiOperation(value = "路口设备状态信息", notes = "路口设备状态信息", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON)
    @GetMapping(value = "crossDeviceStatusInfo", produces = MediaType.APPLICATION_JSON)
    public JsonViewObject crossDeviceStatusInfo() {
        List<CrossDeviceStatusInfoResultVO> list = situationDetectionService.crossDeviceStatusInfo();
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        return jsonViewObject.success(list);
    }

    @ApiOperation(value = "统一事件告警检测", notes = "统一事件告警检测", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON)
    @GetMapping(value = "commonEventWarnNotify", produces = MediaType.APPLICATION_JSON)
    public JsonViewObject commonEventWarnNotify() {
        // 路口报警
        // 事件报警
        // 信号机故障
        Map<String, String> header = buildHeader();
        JsonViewObject jsonViewObject = httpRestUtil.doPost(utcServiceUrl.concat(syncDeviceSatusUrl), header, "", JsonViewObject.class);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            return jsonViewObject.fail("远程服务调用异常".concat(utcServiceUrl.concat(syncDeviceSatusUrl)));
        }
        LocalDateTime now = LocalDateTime.now();
        String formatNow = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss").format(now);
        List<Map<String, Object>> content = (List<Map<String, Object>>) jsonViewObject.getContent();
        Map<String, Object> result = content.get(0);
        String signalId = (String) result.get("signalId");
        String crossId = (String) result.get("crossId");
        Integer faultType = (Integer) result.get("faultType");
        if (Objects.nonNull(faultType) && faultType != 0 && StringUtils.isNotEmpty(crossId)) {
            CommonEventAlarmOutVo commonEventAlarmOutVo = new CommonEventAlarmOutVo();
            commonEventAlarmOutVo.setEventType(3);
            commonEventAlarmOutVo.setEventId(signalId);
            commonEventAlarmOutVo.setEventName("信号机故障");// todo 信号机厂商
            commonEventAlarmOutVo.setEventDesc("信号机故障");
            commonEventAlarmOutVo.setStatus(String.valueOf(faultType));
            commonEventAlarmOutVo.setStartTime(formatNow);

            TBaseCrossInfo tBaseCrossInfo = situationDetectionService.selectCrossInfoById(crossId);
            if (Objects.nonNull(tBaseCrossInfo)) {
                commonEventAlarmOutVo.setCoordinate(tBaseCrossInfo.getLocation());
            }
            jsonViewObject.setContent(commonEventAlarmOutVo);
        } else {
            jsonViewObject.setContent(null);
            jsonViewObject.setMessage("无告警数据");
        }

        return jsonViewObject;
    }
}
