package net.wanji.web.dto;

import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.util.List;

/**
 * @author Kent HAN
 * @date 2023/1/5 10:30
 */
@NoArgsConstructor
@Data
public class SaveSchemeConfigDTO {
    @ApiModelProperty(value = "路口ID", required = true)
    private String crossId;
    @ApiModelProperty(value = "配时方式 1阶段式 2环式", required = true)
    private Integer timeType;
    private List<PhaseScheme> phaseSchemeList; // 相位方案列表
    private List<TimeScheme> timeSchemeList; // 配时方案列表

    @Data
    public static class PhaseScheme {
        @ApiModelProperty(value = "方案号", required = true)
        private String schemeNo;
        @ApiModelProperty(value = "方案名称", required = true)
        private String name;
        private List<Ring> ringList; // 环列表

        @Data
        public static class Ring {
            @ApiModelProperty(value = "环号", required = true)
            private Integer ringNo;
            List<Phase> phaseList; // 相位列表
            public static class Phase {
                @ApiModelProperty(value = "相位序号", required = true)
                private Integer sort;
                @ApiModelProperty(value = "相位号", required = true)
                private String phaseNo;
                private List<Lane> laneList; // 车道列表
                @Data
                public static class Lane {
                    @ApiModelProperty(value = "车道序号，从左车道开始编号11、12、13...", required = true)
                    private Integer sort;
                    @ApiModelProperty(value = "车道方向：1北；2东北；3东；4东南；5南；6西南；7西；8西北", required = true)
                    private Integer dir;
                    @ApiModelProperty(value = "车道转向：1左转；2直行；3右转；4掉头；5左直；6直右；7左直右；8左右；9左转掉头；10直行掉头；11右转掉头；12左直掉头；13直右掉头；14左直右掉头；15左右掉头", required = true)
                    private Integer turn;
                }
            }
        }
    }

    @Data
    public static class TimeScheme {
        @ApiModelProperty(value = "方案号", required = true)
        private String schemeNo;
        @ApiModelProperty(value = "方案名称", required = true)
        private String name;
        @ApiModelProperty(value = "协调相位号", required = true)
        private String coordPhaseNo;
        @ApiModelProperty(value = "周期", required = true)
        private Integer cycle;
        @ApiModelProperty(value = "相位差", required = true)
        private Integer offset;
        private List<Ring> ringList; // 环列表
        @Data
        public static class Ring {
            @ApiModelProperty(value = "环号", required = true)
            private Integer ringNo;
            List<Phase> phaseList; // 相位列表
            @Data
            public static class Phase {
                @ApiModelProperty(value = "相位序号", required = true)
                private Integer sort;
                @ApiModelProperty(value = "相位号", required = true)
                private String phaseNo;
                private List<Lane> laneList; // 车道列表
                @Data
                public static class Lane {
                    @ApiModelProperty(value = "车道方向：1北；2东北；3东；4东南；5南；6西南；7西；8西北", required = true)
                    private Integer dir;
                    @ApiModelProperty(value = "车道转向：1左转；2直行；3右转；4掉头；5左直；6直右；7左直右；8左右；9左转掉头；10直行掉头；11右转掉头；12左直掉头；13直右掉头；14左直右掉头；15左右掉头", required = true)
                    private Integer turn;
                }
                @ApiModelProperty(value = "最小绿灯时间", required = true)
                private Integer minGreenTime = 0;
                @ApiModelProperty(value = "最大绿灯时间", required = true)
                private Integer maxGreenTime = 0;
                @ApiModelProperty(value = "绿灯时间", required = true)
                private Integer greenTime = 0;
                @ApiModelProperty(value = "绿闪时间", required = true)
                private Integer greenFlashTime = 0;
                @ApiModelProperty(value = "黄灯时间", required = true)
                private Integer yellowTime = 0;
                @ApiModelProperty(value = "红灯时间", required = true)
                private Integer redTime = 0;
                @ApiModelProperty(value = "是否黄闪 0否 1是", required = true)
                private Integer yellowFlash = 0;
                @ApiModelProperty(value = "是否红闪 0否 1是", required = true)
                private Integer redFlash = 0;
            }
        }
    }
}